/*
 * Copyright (c) 2005 Silicon Graphics, Inc.
 * All rights reserved.
 *
 *   Michael A. Raymond <mraymond@sgi.com>
 */

/*
 *  This file is part of libuli.
 *
 *  libuli is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 *
 *  libuli is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the Lesser GNU General Public License
 *  along with libuli; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
#include <asm/asmmacro.h>

/*
 * This is architecture specific code for libuli.  It contains the two
 * functions ULI_tao() and ULI_tas() which must do atomic bit operations
 * on the ULI structure's lock variable.
 */
        
/*
 * ULI_tao(int * lock, int flag)
 *
 * Set the flag value in the lock and return the value of the lock
 * before the flag was set.
 */
GLOBAL_ENTRY(ULI_tao)
1:	ld4 r2 = [r32]   // Get the old value
	;;
	or r3 = r2, r33  // OR in the new bit
	mov ar.ccv = r2  // Tell the CPU the value to compare against
	;;
	cmpxchg4.acq r8 = [r32], r3, ar.ccv      // Compare & Exchange
	;;
	cmp4.ne p8,p0 = r2,r8    // Old value matches the attempted swapped one?
	;;
(p8)	br.cond.spnt.few 1b      // If not a match then try again
	br.ret.sptk.many rp      // Return the result
END(ULI_tao)

/*
 * ULI_tas(int * lock, int new_val)
 *
 * Atomically swap *lock and new_val.
 */
GLOBAL_ENTRY(ULI_tas)
	xchg4 r8 = [r32], r33    // Atomically swap values
	br.ret.sptk.many rp      // Return the result
END(ULI_tas)
