/*
 * Copyright (c) 2005 Silicon Graphics, Inc.
 * All rights reserved.
 *
 *   Michael A. Raymond <mraymond@sgi.com>
 */

/*
 *  This file is part of the ULI sample applications.
 *
 *  seg_test is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  seg_test is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with uli_sample; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <uli.h>

/*
 * This program tests the ULI safety mechanisms by having the handler
 * segfault.  The program should exit through the fatal signal SIGSEGV.
 */

#define ULI_CPU 0
void * ULIreg;
volatile int uli_data = 0;

/*
 * Modify a global variable so the main program can see that we ran.
 */
static void
uli_handler(void * arg_data)
{
	int * data = (int*)arg_data;
	int * seg_data = NULL;

	/* Update the global variable */
	*data = 2;

	/* Force a segfault */
	*seg_data = 0x30;

	return;
}

int
main(int argc, char * argv[])
{
	int line;

	/* Make sure an IRQ line was requested */
	if (argc < 2) {
		fprintf(stderr, "Must supply an IRQ line to use\n");
		return 1;
	}

	line = atoi(argv[1]);

	/* Lock down all the memory */
	if (mlockall(MCL_CURRENT | MCL_FUTURE) < 0) {
		perror("mlockall");
		return 1;
	}

	/* Register the handler */
	printf("Calling ULI_register\n");
	ULIreg = ULI_register_irq(ULI_CPU, uli_handler, (void*)&uli_data, 0,
				     NULL, 0, line);

	if (!ULIreg) {
		fprintf(stderr, "Problem with ULI_register_irq");
		return 1;
	}
	printf("Did registration\n");

	/*
	 * We loop waiting for the handler to run and change the data
	 * we passed it.
	 */
	while (1) {
		sleep(2);
		if (uli_data != 0) {
			printf("tester %d\n",
				uli_data);
			break;
		}
	}

	printf("Done\n");

	sleep(2);
	return 0;
}
